@@ -20,7 +20,7 @@ from kabaret.subprocess_manager.flow import RunAction
 
 from .maputils import SimpleCreateAction, ClearMapAction
 
-from .site import SyncMap, Request, RequestAs
+from .site import SyncMap, Request, RequestAs, UploadRevision
 from .runners import LaunchSessionWorker, CHOICES, CHOICES_ICONS
 from .kitsu import KitsuTaskStatus
 
@@ -208,21 +208,29 @@ class PublishFileAction(LaunchSessionWorker):
         return self.oid()
 
     def launcher_exec_func_name(self):
-        return "process_after_publish"
+        return "_process_revision"
     
-    def process_after_publish(self):
-        if self.upload_after_publish.get():
-            self.upload_published_revision()
+    def _target_file(self):
+        return self._file
+    
+    def _revision_to_process(self):
+        return self._target_file().get_head_revision()
+    
+    def _process_revision(self):
+        file = self._target_file()
+        rev = self._revision_to_process()
         
-        if self._file.format.get() == 'blend':
-            self.save_blend_dependencies()
+        if self.upload_after_publish.get():
+            self._upload(rev)
+        
+        if file.format.get() == 'blend':
+            self._save_blend_dependencies(rev)
     
-    def upload_published_revision(self):
-        head = self._file.get_head_revision()
+    def _upload(self, revision):
         current_site = self.root().project().get_current_site()
 
         upload_job = current_site.get_queue().submit_job(
-            emitter_oid=head.oid(),
+            emitter_oid=revision.oid(),
             user=self.root().project().get_user(),
             studio=current_site.name(),
             job_type='Upload',
@@ -231,13 +239,12 @@ class PublishFileAction(LaunchSessionWorker):
         process_jobs_action = self.root().project().admin.process_jobs
         process_jobs_action._process(upload_job)
     
-    def save_blend_dependencies(self):
+    def _save_blend_dependencies(self, revision):
         from blender_asset_tracer import trace, bpathlib
         from pathlib import Path
         import collections
 
-        head = self._file.get_head_revision()
-        path = Path(head.get_path())
+        path = Path(revision.get_path())
         report = collections.defaultdict(list)
 
         for usage in trace.deps(path):
@@ -245,26 +252,27 @@ class PublishFileAction(LaunchSessionWorker):
             asset_path = str(usage.asset_path).replace('//', '')
             report[str(filepath)].append(asset_path)
         
-        head.dependencies.set(dict(report))
+        revision.dependencies.set(dict(report))
 
     def run(self, button):
         if button == "Cancel":
             return
-
-        self._file.lock()
+        
+        target_file = self._target_file()
+        
+        target_file.lock()
         keep_editing = (button == "Keep editing")
-        published_revision = self._file.publish(
+        published_revision = target_file.publish(
             comment=self.comment.get(),
             keep_editing=keep_editing,
         )
 
         if not keep_editing:
-            self._file.set_current_user_on_revision(published_revision.name())
-            self._file.unlock()
+            target_file.set_current_user_on_revision(published_revision.name())
+            target_file.unlock()
 
         published_revision.make_current.run(None)
-
-        self._file.touch()
+        target_file.touch()
 
         super(PublishFileAction, self).run(None)
 
@@ -330,6 +338,7 @@ class Revision(flow.Object):
     create_working_copy = flow.Child(CreateWorkingCopyFromRevision)
     request = flow.Child(Request)
     request_as = flow.Child(RequestAs)
+    upload = flow.Child(UploadRevision)
     compute_hash_action = flow.Child(ComputeRevisionHash)
     check_hash = flow.Child(CheckRevisionHash)
     dependencies = flow.Param("").ui(editor='textarea', editable=False)
@@ -1234,6 +1243,11 @@ bpy.ops.lfs.playblast(do_render=True, filepath='%s', studio='%s', project='%s',
 
 
 class PublishAndRenderPlayblastAction(PublishFileAction):
+    
+    use_simplify = flow.SessionParam(False).ui(
+        tooltip="Use low-definition rigs",
+        editor='bool',
+    )
 
     def allow_context(self, context):
         return (
@@ -1250,6 +1264,7 @@ class PublishAndRenderPlayblastAction(PublishFileAction):
         published_revision = self._file.get_head_revision()
 
         self._file.render_playblast.revision_name.set(published_revision.name())
+        self._file.render_playblast.use_simplify.set(self.use_simplify.get())
 
         return self._file.render_playblast.run("Render")
 
@@ -1263,6 +1278,9 @@ class FileRevisionNameChoiceValue(flow.values.ChoiceValue):
         return self.action._file
 
     def choices(self):
+        if self.get_file() is None:
+            return []
+        
         return self.get_file().get_revision_names(
             sync_status='Available',
             published_only=True
@@ -1294,10 +1312,16 @@ class UploadPlayblastToKitsu(flow.Action):
     
     def __init__(self, parent, name):
         super(UploadPlayblastToKitsu, self).__init__(parent, name)
-        kitsu_bindings = self.root().project().kitsu_bindings()
-        file_settings = get_contextual_dict(self._file, 'settings')
-        entity_data = kitsu_bindings.get_entity_data(file_settings)
-        self._kitsu_entity = kitsu_bindings.get_kitsu_entity(entity_data)
+        self._kitsu_entity = None
+    
+    def _ensure_kitsu_entity(self):
+        if self._kitsu_entity is None:
+            kitsu_bindings = self.root().project().kitsu_bindings()
+            file_settings = get_contextual_dict(self._file, 'settings')
+            entity_data = kitsu_bindings.get_entity_data(file_settings)
+            self._kitsu_entity = kitsu_bindings.get_kitsu_entity(entity_data)
+        
+        return self._kitsu_entity
     
     def allow_context(self, context):
         kitsu_config = self.root().project().admin.kitsu
@@ -1317,64 +1341,72 @@ class UploadPlayblastToKitsu(flow.Action):
         
         # Check if the file is linked to a Kitsu entity
         task_type = self.target_task_type.get()
+        kitsu_entity = self._ensure_kitsu_entity()
         
-        if self._kitsu_entity is None or task_type is None:
-            self.message.set((
-                "<h2><font color=#D5000D>The Kitsu entity %s belongs to "
-                "couldn't be detected.</font></h2>Please contact the "
-                "support on the chat." % self._file.display_name.get()
-            ))
+        msg = "<h2>Upload playblast to Kitsu</h2>"
+        
+        if kitsu_entity is None or task_type is None:
+            msg += (
+                "<h3><font color=#D5000D>The Kitsu entity %s belongs to "
+                "couldn't be detected. Please contact the "
+                "support on the chat.</font></h3>" % self._file.display_name.get()
+            )
+            self.message.set(msg)
             return ['Cancel']
         
         # Check if current user is assigned to a Kitsu task this file is made for
         kitsu_api = self.root().project().kitsu_api()
         user = kitsu_api.get_user()
-        task = kitsu_api.get_task(self._kitsu_entity, task_type)
+        task = kitsu_api.get_task(kitsu_entity, task_type)
         
         if user is None:
-            self.message.set((
-                "<h2><font color=#D5000D>It seems you (%s) have no "
-                "user profile on Kitsu.</font></h2>Please contact the "
-                "support on the chat" % self.root().project().get_user()
-            ))
+            msg += (
+                "<h3><font color=#D5000D>It seems you (%s) have no "
+                "user profile on Kitsu. Please contact the "
+                "support on the chat.</font></h3>" % self.root().project().get_user()
+            )
+            self.message.set(msg)
             return ['Cancel']
         
         if task is None:
-            self.message.set((
-                "<h2><font color=#D5000D>This file is not linked to any "
-                "task on Kitsu.</font></h2>"
-            ))
+            msg += (
+                "<h3><font color=#D5000D>This file is not linked to any "
+                "task on Kitsu.</font></h3>"
+            )
+            self.message.set(msg)
             return ['Cancel']
         
         # Check if user is assigned to the task or have sufficient rights
-        valid_user_role = (user['role'] in ['admin', 'manager'])
         is_assigned = kitsu_api.user_is_assigned(user, task)
+        user_role = user.get('role', None)
         
-        if not is_assigned and not valid_user_role:
-            self.message.set((
-                "<h2><font color=#D5000D>You (%s) are not assigned to "
-                "the task this file has been created for.</font></h2>"
-                % self.root().project().get_user()
-            ))
-            return ['Cancel']
-        
-        msg = "<h2>Upload playblast to Kitsu</h2>"
-        user_roles = {
-            'admin': 'studio manager',
-            'manager': 'supervisor'
-        }
-        
-        if valid_user_role and not is_assigned:
-            msg += (
-                "As %s, you can upload a preview for this file."
-                % user_roles[user['role']]
-            )
+        if not is_assigned:
+            if not user_role in ['admin', 'manager']:
+                msg += (
+                    "<h3><font color=#D5000D>You (%s) are not assigned to "
+                    "the task this file has been created for.</font></h3>"
+                    % self.root().project().get_user()
+                )
+                self.message.set(msg)
+                return ['Cancel']
+            else:
+                user_roles = {
+                    'admin': 'studio manager',
+                    'manager': 'supervisor'
+                }
+                msg += (
+                    "<h3>As %s, you can upload a preview for this file.</h3>"
+                    % user_roles[user_role]
+                )
+
         self.message.set(msg)
         
         return ['Upload', 'Cancel']
     
     def compute_child_value(self, child_value):
-        if self._kitsu_entity is None:
+        kitsu_entity = self._ensure_kitsu_entity()
+        
+        if kitsu_entity is None:
             child_value.set(None)
             return
         
@@ -1388,7 +1420,7 @@ class UploadPlayblastToKitsu(flow.Action):
             self.target_task_type.set(task_type)
         elif child_value is self.current_task_status:
             task_status = kitsu_api.get_task_current_status(
-                self._kitsu_entity,
+                kitsu_entity,
                 self.target_task_type.get()
             )
             self.current_task_status.set(task_status)
@@ -1401,7 +1433,7 @@ class UploadPlayblastToKitsu(flow.Action):
         kitsu_api = self.root().project().kitsu_api()
         
         success = kitsu_api.upload_preview(
-            kitsu_entity=self._kitsu_entity,
+            kitsu_entity=self._ensure_kitsu_entity(),
             task_type_name=self.target_task_type.get(),
             task_status_name=self.target_task_status.get(),
             file_path=self._file.get_revision(self.revision_name.get()).get_path(),
@@ -1690,7 +1722,7 @@ class TrackedFile(File):
             shutil.copy2(source_path, working_copy_path)
         except OSError:
             self.root().session().log_error(
-                "Copy of template '{}' failed\n\n==> TRACEBACK\n".format(file_path)
+                "Copy of source file '{}' failed\n\n==> TRACEBACK\n".format(source_path)
             )
             raise
 
@@ -1928,8 +1960,8 @@ class PublishIntoFile(PublishFileAction):
         label="Source revision"
     )
     target_file = flow.Connection(ref_type=ResetableTrackedFileRef).watched()
-    revision_name = flow.SessionParam("").watched()
-    comment = flow.SessionParam("")
+    revision_name = flow.Param("").watched()
+    comment = flow.Param("")
     upload_after_publish = flow.BoolParam(False)
 
     def get_buttons(self):
@@ -1993,6 +2025,16 @@ class PublishIntoFile(PublishFileAction):
         self.message.set(msg)
         return True
     
+    def _target_file(self):
+        return self.target_file.get()
+    
+    def _revision_to_process(self):
+        revision_name = self.revision_name.get()
+        if not revision_name:
+            revision_name = self.source_revision_name.get()
+
+        return self._target_file().get_revision(revision_name)
+
     def child_value_changed(self, child_value):
         self.message.set("<h2>Publish from an existing file</h2>")
 
@@ -2021,6 +2063,8 @@ class PublishIntoFile(PublishFileAction):
             return self.get_result(close=False)
         
         revision_name = self.revision_name.get()
+        if not revision_name:
+            revision_name = self.source_revision_name.get()
         
         # Check choosen revision name
         if not self.check_revision_name(revision_name):
@@ -2037,6 +2081,7 @@ class PublishIntoFile(PublishFileAction):
             source_path=source_revision.get_path(),
             comment=self.comment.get(),
         )
+        target_file.make_current(publication)
         target_file.unlock()
         target_file._map.touch()
 
