@@ -166,7 +166,7 @@ class KitsuAPIWrapper(flow.Object):
             self.root().session().log_error((
                 f"No user '{project_user_name}' registered in this project"
             ))
-            return False
+            return None
         
         user_kitsu_id = user.kitsu_id.get()
         kitsu_user = gazu.person.get_person_by_desktop_login(user_kitsu_id)
@@ -198,15 +198,7 @@ class KitsuAPIWrapper(flow.Object):
         # Get task
         task = self.get_task(kitsu_entity, task_type_name)
         
-        if task is None:
-            return False
-        
-        # Check if uploading user is assigned to the task
-        if not self.user_is_assigned(user, task):
-            self.root().session().log_error((
-                f"User '{user_name}' is not assigned to task '{task_type_name}' "
-                "of entity '%s'." % kitsu_entity['name']
-            ))
+        if task is None or user is None:
             return False
         
         # Add comment with preview
